#ifndef __SCENE_H
#define __SCENE_H

#include "vector.h"
#include "color.h"
#include "light.h"
#include "objects.h"
#include "ray.h"

#include <vector>
#include <math.h>

// stala ograniczajaca promien odgornie
const float maxray = 1000000.0;

/*
Pomocniczy agragat do przechowywania tymczasowej informacji o aktualnie
badanym punkcie powierzchni
*/

class ShInfo
{
public:
	Vector	normal;
	Vector	reflect;
	Vector	point;
	Object *object;
	Ray *ray;
};


/*
Rdzen calego programu - klasa sceny laczy wszystkie informacje o obiektach,
swiatlach, kamerze itd. oraz za pomoca metody Trace liczy obrazek.

Szereg metod dostepnych z zewnatrz sluzy do ustawienia wszystkich
niezbednych parametrow sceny.
*/



class Scene
{
private:
	std::vector<Object*>	objects; // lista obiektow sceny
	std::vector<Light*>	lights; // lista swiatel sceny

	Color		ambient; // kolor tla

	unsigned	xres, yres; // rozdzielczosc

	float		FOV; // pole widzenia (kat)
	float		focus; // odleglosc kamery od ekranu
	float		aspect; // wsp. proporcjonalnosci

	Vector		campos; // pozycja kamery
	Vector		camdir; // kierunek patrzenia
	float		roll; // obrot kamery wokol osi patrzenia

	Color		fog; // kolor mgly
	float		dmax, dmin; // zasieg mgly
	int			fogen; // czy mgla jest aktywna?

	int			maxdeep; // maksymalna ilosc zaglebien rekurencji

	char		name[255]; // nazwa pliku


	Object *FindNearest(const Ray &l, float &t); // znalezienie najblizszego przeciecia 
	Color CalcColor(const ShInfo* info, int deep); // obliczenie koloru w punkcie z zadana liczba zaglebien
	float CalcFocus(); // obliczenie parametru focus
	float CalcFog(float depth); // obliczenie mgly w zadanej odleglosci od obserwatora

public:

	Scene();


	void SetAmbient(const Color &col) {ambient = col;} // ustawia kolor oswietlenia rozproszonego
	void SetFog(float min, float max, Color col); // ustawia parametry mgly
	void FogEn() {fogen = 1;} // wlacza mgle
	void FogDis() {fogen = 0;} // wylacza mgle
	void SetAspect(float a) {aspect = a;} // ustawia stosunek pikseli (szer./wys.)
	void SetH(unsigned H) {xres = H;} // ustawia szerokosc obrazka
	void SetV(unsigned V) {yres = V;} // ustawia wysokosc obrazka
	void SetFOV(unsigned FOV); // ustawia pole widzenia
	void SetFile(const char *fname); // ustawia nazwe pliku do renderowania
	void SetCam(const Vector &pos, const Vector &dir); // ustawia kamere
	void SetRoll(float r) {roll = r;} // ustawia obrot kamery
	void SetDeep(int max) {maxdeep = max;} // ustawia maksymalna wartosc zaglebienia

	int AddObj(Object *obj); // dodanie obiektu do sceny
	int AddLight(Light *lt); // dodanie swiatla do sceny

	int Trace(); // generuje scene
};

#endif


